﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SpawnerOfEnemies : MonoBehaviour {

	public enum SpawnState { SPAWNING, WAITING, COUNTING };

	[System.Serializable]
	public class Wave{
		public string name;
		public Transform enemy;
		public int count;
		public float spawnRate;
		public int vel;
	}

	public Wave[] waves;
	private int nextWave = 0;

	public Transform[] spawnPoints;

	public float spawnTimeBetween = 5f;
	public float waveCountdown;

	private float searchCountdown = 1f;

	private SpawnState state = SpawnState.COUNTING;

	void Start(){
		
		if (spawnPoints.Length == 0) {
			Debug.LogError ("No spawn points referenced.");
		}

		waveCountdown -= spawnTimeBetween;
	}

	void Update (){
		if (state == SpawnState.WAITING) {
			if (!EnemyIsAlive ()) {
				//Begin a new wave of enemies
				WaveCompleted();
				return;
			} else {
				return;
			}
		}

		if (waveCountdown <= 0) {
			if (state != SpawnState.SPAWNING) {
				StartCoroutine (SpawnWave (waves [nextWave]));
			}
		} else {
			waveCountdown -= Time.deltaTime;
		}
	}

	void WaveCompleted() {
		Debug.Log ("Wave Completed!");

		state = SpawnState.COUNTING;

		waveCountdown = spawnTimeBetween;

		if (nextWave + 1 > waves.Length - 1) {
			nextWave = 0;
			Debug.Log ("All waves complete! LOOPING...!!!");
		} else {
			nextWave++;
		}
	}

	bool EnemyIsAlive (){
		searchCountdown -= Time.deltaTime;
		if (searchCountdown <= 0f){
			searchCountdown = 1f;	
			if (GameObject.FindGameObjectWithTag ("Enemy") == null) {
				return false;
			}
			return true;
		}
		return true;
	}

	IEnumerator SpawnWave (Wave _wave) {
		Debug.Log ("Spawning Wave: " + _wave.name);

		state = SpawnState.SPAWNING;

		//spawning
		for (int i = 0; i < _wave.count; i++){
			SpawnEnemy (_wave.enemy.gameObject, _wave.vel);
			float rand = Random.Range(0.01f, _wave.spawnRate);

			yield return new WaitForSeconds (_wave.spawnRate - rand);
		}

		state = SpawnState.WAITING;

		yield break;
	}

	void SpawnEnemy (GameObject _enemy, int velocidad){
		//Debug.Log ("Spawning Enemy: " + _enemy.name);

		Transform _sp = spawnPoints [Random.Range (0, spawnPoints.Length)];
		GameObject enemy = Instantiate (_enemy, _sp.position, _sp.rotation) as GameObject;
		if (enemy.GetComponent<EnemyCode> () != null) {
			enemy.GetComponent<EnemyCode> ().moveSpeed = velocidad;
		}
		if (enemy.GetComponent<EnemyCodeTwo> () != null) {
			enemy.GetComponent<EnemyCodeTwo> ().moveSpeed = velocidad;
		}
	}


}
